function se=gaussep_inst(x,A,wlen,lf,lt)

% GAUSSEP_INST Maximum Likelihood separation of an instantaneous mixture
% by local Gaussian modeling of the sources, with windowed covariance
% estimates
%
% Inputs:
% x: I x T mixture signal
% A: I x J real-valued mixing matrix
% wlen: STFT window length
% lf: half-width of frequency neighborhood
% lt: half-width of time neighborhood
%
% Outputs:
% se: I x T x J estimated source images
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2009 Emmanuel Vincent
% This software is distributed under the terms of the GNU Public License
% version 3 (http://www.gnu.org/licenses/gpl.txt)
% If you find it useful, please cite the following reference:
% Emmanuel Vincent, Simon Arberet and Rémi Gribonval, "Underdetermined
% instantaneous audio source separation via local Gaussian modeling," in 
% Proc. Int. Conf. on Independent Component Analysis and Signal Separation, 
% pp. 775-782, 2009.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%% Errors and warnings %%%
if nargin<5, error('Not enough input arguments.'); end
[I,T]=size(x);
[Ia,J]=size(A);
if I~=Ia, error('The number of colums of the mixing matrix must be equal to the number of mixture channels.'); end
if (I>2)&&(exist('fmincon','file')~=2), error('The MATLAB Optimization Toolbox is not available.'); end
if I>J, error('The number of sources must be larger than the number of mixture channels.'); end
N=ceil(T/wlen*2);

%%% Time-frequency transformation %%%
X=zeros(wlen/2+1,N,I);
for i=1:I,
    X(:,:,i)=stft(x(i,:),wlen);
end
Se=zeros(wlen/2+1,N,J);

%%% Definition of algorithm constants %%%
% Time-frequency windows
winf=hanning(2*lf-1);
wint=hanning(2*lt-1).';
% Minimum and maximum source activity patterns
act=zeros(2^J,J);
for p=1:J,
    act(:,p)=reshape(repmat([0 1],[2^(J-p) 2^(p-1)]),2^J,1);
end
actmin=logical(act(sum(act,2)==I,:));
npatmin=size(actmin,1);
Jmax=I*(I+1)/2;
actmax=logical(act(sum(act,2)==Jmax,:));
npatmax=size(actmax,1);
% Transformation from source variances to mixture covariance
B=zeros(I^2,J);
for j=1:J,
    AAt=A(:,j)*A(:,j).';
    B(:,j)=AAt(:);
end
Binv=zeros(Jmax,I^2,npatmax);
for p=1:npatmax,
    Binv(:,:,p)=pinv(B(:,actmax(p,:)));
end
% Inverse mixing matrices
Ainv=zeros(I,I,npatmin);
for p=1:npatmin,
    Ainv(:,:,p)=pinv(A(:,actmin(p,:)));
end

%%% Separation over individual time-frequency bins %%%
options=optimset('GradObj','on','Hessian','on','TolX',1e-5,'TolFun',1e-5,'MaxIter',50,'LargeScale','on','Display','off');
for f=1:wlen/2+1,
    for t=1:N,
        % Local mixture covariance
        indf=max(1,f-lf+1):min(wlen/2+1,f+lf-1);
        indt=max(1,t-lt+1):min(N,t+lt-1);
        nind=length(indf)*length(indt);
        XX=reshape(X(indf,indt,:),nind,I);
        wei=reshape(winf(indf-f+lf)*wint(indt-t+lt),nind,1);
        Cx=real(XX'*(XX.*(wei*ones(1,I))))/sum(wei);
        % Maximum activity solutions
        vact=zeros(Jmax,npatmax);
        for p=1:npatmax,
            vact(:,p)=Binv(:,:,p)*Cx(:);
        end
        isexact=find(all(vact >= 0));
        if ~isempty(isexact),
            lnorm=zeros(1,length(isexact));
            for p=1:length(isexact),
                sigma=diag(sqrt(vact(:,isexact(p))));
                s=sigma*pinv(A(:,actmax(isexact(p),:))*sigma)*reshape(X(f,t,:),I,1);
                lnorm(p)=prod(abs(s));
            end
            [val,ind]=min(lnorm);
            v=zeros(J,1);
            v(actmax(isexact(ind),:))=vact(:,isexact(ind));
        else
            % Minimum activity solutions
            vact=zeros(I,npatmin);
            crit=zeros(1,npatmin);
            for p=1:npatmin,
                Cs=Ainv(:,:,p)*Cx*Ainv(:,:,p).';
                vact(:,p)=diag(Cs);
                crit(p)=-.5*log(det(Cs)/prod(vact(:,p)));
            end
            [crit,ind]=min(crit);
            v=zeros(J,1);
            v(actmin(ind,:))=vact(:,ind);
            % Iterative optimization
            if I>2,
                vinit=v;
                [crit,grad]=diverg(vinit,Cx,A);
                if any(grad(~vinit) < 0) && ~isinf(crit),   % Avoid possible infinite loop in fmincon caused by numerical optimality check
                    v=fmincon(@(v) diverg(v,Cx,A),vinit,[],[],[],[],zeros(J,1),inf(J,1),[],options);
                end
            end
        end
        % Multichannel Wiener filter
        sigma=diag(sqrt(v));
        Se(f,t,:)=sigma*pinv(A*sigma)*reshape(X(f,t,:),I,1);
    end
end

%%% Inverse time-frequency transformation %%%
se=zeros(I,N*wlen/2,J);
for j=1:J,
    se(:,:,j)=A(:,j)*istft(Se(:,:,j));
end
se=se(:,1:T,:);

return;



function [crit,grad,hess]=diverg(v,Cx,A)

[I,J]=size(A);
% Kullback Leibler divergence on the mixture covariance
R=A*diag(v)*A.';
Rinv=inv(R);
P=Rinv*Cx;
crit=.5*(trace(P)-log(det(P))-I);
% Gradient
if nargout > 1,
    grad=zeros(J,1);
    E=P-eye(I);
    Pd=zeros(I,I,J);
    for j=1:J,
        Pd(:,:,j)=Rinv*A(:,j)*A(:,j).';
        grad(j)=-.5*trace(E*Pd(:,:,j));
    end
    % Hessian
    if nargout > 2,
        hess=zeros(J);
        E=2*P-eye(I);
        for j1=1:J,
            for j2=1:j1,
                hess(j1,j2)=.5*trace(E*Pd(:,:,j2)*Pd(:,:,j1));
            end
        end
        hess=hess+hess.'-diag(diag(hess));
    end
end

return;