function se=ssdp_inst(x,A,wlen,lf,lt)

% SSDP_INST Maximum Likelihood separation of an instantaneous mixture
% by local Gaussian modeling of the sources, with windowed covariance
% estimates, under the constraint that the number of active sources is
% equal to the number of mixture channels
%
% Inputs:
% x: I x T mixture signal
% A: I x J real-valued mixing matrix
% wlen: STFT window length
% lf: half-width of frequency neighborhood
% lt: half-width of time neighborhood
%
% Outputs:
% se: I x T x J estimated source images
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Copyright 2009 Emmanuel Vincent
% This software is distributed under the terms of the GNU Public License
% version 3 (http://www.gnu.org/licenses/gpl.txt)
% If you find it useful, please cite the following reference:
% Emmanuel Vincent, Simon Arberet and Rémi Gribonval, "Underdetermined
% instantaneous audio source separation via local Gaussian modeling," in 
% Proc. Int. Conf. on Independent Component Analysis and Signal Separation, 
% pp. 775-782, 2009.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%% Errors and warnings %%%
if nargin<5, error('Not enough input arguments.'); end
[I,T]=size(x);
[Ia,J]=size(A);
if I~=Ia, error('The number of colums of the mixing matrix must be equal to the number of mixture channels.'); end
if I>J, error('The number of sources must be larger than the number of mixture channels.'); end
N=ceil(T/wlen*2);

%%% Time-frequency transformation %%%
X=zeros(wlen/2+1,N,I);
for i=1:I,
    X(:,:,i)=stft(x(i,:),wlen);
end
Se=zeros(wlen/2+1,N,J);

%%% Definition of algorithm constants %%%
% Time-frequency windows
winf=hanning(2*lf-1);
wint=hanning(2*lt-1).';
% Minimum source activity patterns
act=zeros(2^J,J);
for p=1:J,
    act(:,p)=reshape(repmat([0 1],[2^(J-p) 2^(p-1)]),2^J,1);
end
actmin=logical(act(sum(act,2)==I,:));
npatmin=size(actmin,1);
% Inverse mixing matrices
Ainv=zeros(I,I,npatmin);
for p=1:npatmin,
    Ainv(:,:,p)=pinv(A(:,actmin(p,:)));
end

%%% Separation over individual time-frequency bins %%%
for f=1:wlen/2+1,
    for t=1:N,
        % Local mixture covariance
        indf=max(1,f-lf+1):min(wlen/2+1,f+lf-1);
        indt=max(1,t-lt+1):min(N,t+lt-1);
        nind=length(indf)*length(indt);
        XX=reshape(X(indf,indt,:),nind,I);
        wei=reshape(winf(indf-f+lf)*wint(indt-t+lt),nind,1);
        Cx=real(XX'*(XX.*(wei*ones(1,I))))/sum(wei);
        % Minimum activity solution
        vact=zeros(I,npatmin);
        crit=zeros(1,npatmin);
        for p=1:npatmin,
            Cs=Ainv(:,:,p)*Cx*Ainv(:,:,p).';
            crit(p)=-.5*log(det(Cs)/prod(diag(Cs)));
        end
        [crit,ind]=min(crit);
        % Local mixing inversion
        Se(f,t,actmin(ind,:))=Ainv(:,:,ind)*reshape(X(f,t,:),I,1);
    end
end

%%% Inverse time-frequency transformation %%%
se=zeros(I,N*wlen/2,J);
for j=1:J,
    se(:,:,j)=A(:,j)*istft(Se(:,:,j));
end
se=se(:,1:T,:);

return;
